@@ -153,9 +153,14 @@ Erstelle temporäre Datei.</translation>
     <message>
         <source>More info: http://mnemosyne-proj.sourceforge.net
 </source>
-        <translation>Mehr Informationen unter: http://mnemosyne-proj.sourceforge.net
+        <translation type="obsolete">Mehr Informationen unter: http://mnemosyne-proj.sourceforge.net
 </translation>
     </message>
+    <message>
+        <source>More info: http://mnemosyne-proj.org
+</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>AboutFrm</name>
