@@ -12,54 +12,56 @@ from export_dlg import *
 from add_items_dlg import *
 from edit_item_dlg import *
 from clean_duplicates import *
+from statistics_dlg import *
 from edit_items_dlg import *
 from activate_categories_dlg import *
 from config_dlg import *
 from product_tour_dlg import *
+from tip_dlg import *
+from about_dlg import *
 from sound import *
+from card_prop import *
+from message_boxes import *
 from mnemosyne.core import *
 
+prefix = os.path.dirname(__file__)
+
 
 
 ##############################################################################
 #
-# messageUnableToSave
-#
-#  Note: operator+ would be nicer than append, but the PyQt version we use
-#        under Windows does not support this.
+# Tooltip texts
 #
 ##############################################################################
 
-def messageUnableToSave(fileName):
+tooltip = [["","","","","",""],["","","","","",""]]
+
+def install_tooltip_strings(self):
+
+    global tooltip
     
-    QMessageBox.critical(None,
-                         qApp.translate("Mnemosyne", "Mnemosyne"),
-                         qApp.translate("Mnemosyne", "Unable to save file:")\
-                         .append(QString("\n" + fileName)),
-                         qApp.translate("Mnemosyne", "&OK"),
-                         "", "", 0, -1)
+    tooltip[0][0] = \
+        self.trUtf8("You don't remember this card yet.")
+    tooltip[0][1] = \
+        self.trUtf8("Like '0', but it's getting more familiar.").append(\
+        self.trUtf8(" Show it less often."))
+    tooltip[0][2] = tooltip[0][3] = tooltip[0][4] = tooltip[0][5] = \
+        self.trUtf8("You've memorised this card now,").append(\
+        self.trUtf8(" and will probably remember it for a few days."))
 
-
-
-##############################################################################
-#
-# queryOverwriteFile
-#
-##############################################################################
-
-def queryOverwriteFile(fileName):
-    
-    status = QMessageBox.warning(None,
-                                 qApp.translate("Mnemosyne", "Mnemosyne"),
-                                 qApp.translate("Mnemosyne", "File exists: ")\
-                                 .append(QString("\n" + fileName)),
-                                 qApp.translate("Mnemosyne", "&Overwrite"),
-                                 qApp.translate("Mnemosyne", "&Cancel"),
-                                 "", 1, -1)
-    if status == 0:
-        return True
-    else:
-        return False
+    tooltip[1][0] = tooltip[1][1] = \
+        self.trUtf8("You have forgotten this card completely.")
+    tooltip[1][2] = \
+        self.trUtf8("Barely correct answer. The interval was way too long.")
+    tooltip[1][3] = \
+        self.trUtf8("Correct answer, but with much effort.").append(\
+        self.trUtf8(" The interval was probably too long."))
+    tooltip[1][4] = \
+        self.trUtf8("Correct answer, with some effort.").append(\
+        self.trUtf8(" The interval was probably just right."))
+    tooltip[1][5] = \
+        self.trUtf8("Correct answer, but without any").append(\
+        self.trUtf8(" difficulties. The interval was probably too short."))
 
 
 
@@ -78,12 +80,17 @@ class MainDlg(MainFrm):
     ##########################################################################
     
     def __init__(self, filename, parent = None,name = None,fl = 0):
+        
         MainFrm.__init__(self,parent,name,fl)
 
         self.state = "EMPTY"
         self.item = None
         
-        self.shrink = True
+        if sys.platform == 'darwin':
+            # Solves bug that this doesn't become False on resize.
+            self.shrink = False
+        else:
+            self.shrink = True
 
         self.q_sound_played = False
         self.a_sound_played = False
@@ -96,23 +103,47 @@ class MainDlg(MainFrm):
         self.statusBar().addWidget(self.notmem,0,1)
         self.statusBar().addWidget(self.all,0,1)
         self.statusBar().setSizeGripEnabled(0)
-        
+
+        if sys.platform == 'darwin':
+            
+            # The size grip cannot be disabled for OSX, this will 
+            # add extra spacing to prevent part of self.all getting
+            # obscured.
+            
+            osx_spacing = QLabel("   ", self.statusBar())
+            self.statusBar().addWidget(osx_spacing,0,1)
+
+        self.grade_buttons = []
+
+        self.grade_buttons.append(self.grade_0_button)
+        self.grade_buttons.append(self.grade_1_button)
+        self.grade_buttons.append(self.grade_2_button)
+        self.grade_buttons.append(self.grade_3_button) 
+        self.grade_buttons.append(self.grade_4_button)
+        self.grade_buttons.append(self.grade_5_button)
+
+        try:
+            run_plugins()
+        except MnemosyneError, e:
+            messagebox_errors(self, e)
+                
         if filename == None:
             filename = get_config("path")
-        
-        status = load_database(filename)
-        
-        if status == False:
-            QMessageBox.critical(None,
-                    self.trUtf8("Mnemosyne"),
-                    self.trUtf8("Unable to load file. Creating a tmp database."),
-                    self.trUtf8("&OK"), QString(), QString(), 0, -1)
+            
+        try:
+            load_database(filename)
+        except MnemosyneError, e:
+            messagebox_errors(self, LoadErrorCreateTmp())
             filename = os.path.join(os.path.split(filename)[0],"___TMP___.mem")
             new_database(filename)
         
         self.newQuestion()
         self.updateDialog()
 
+        self.timer = QTimer(self)
+        self.connect(self.timer, SIGNAL("timeout()"), soundmanager.update)
+        self.timer.start(250, False)
+
     ##########################################################################
     #
     # resizeEvent
@@ -134,16 +165,16 @@ class MainDlg(MainFrm):
 
         pause_thinking()
 
-        path = os.path.join(os.path.expanduser("~"), ".mnemosyne")
-        out = unicode(QFileDialog.getSaveFileName(path, "*.mem"))\
-              .encode("utf-8")
+        out = unicode(QFileDialog.getSaveFileName(basedir,
+                 self.trUtf8("Mnemosyne databases (*.mem)"), self, None,\
+                 self.trUtf8("New")))
         if out != "":
             
             if out[-4:] != ".mem":
                 out += ".mem"
                 
             if os.path.exists(out):
-                if not queryOverwriteFile(out):
+                if not queryOverwriteFile(self, out):
                     unpause_thinking()
                     return
 
@@ -167,27 +198,23 @@ class MainDlg(MainFrm):
 
         pause_thinking()
                 
-        oldPath = get_config("path")
+        oldPath = expand_path(get_config("path"))
         out = unicode(QFileDialog.getOpenFileName(oldPath,\
-                                                  "*.mem")).encode("utf-8")
+                 self.trUtf8("Mnemosyne databases (*.mem)"), self))
         if out != "":
 
-            status = unload_database()
-            if status == False:
-                messageUnableToSave(oldPath)
+            try:
+                unload_database()
+            except MnemosyneError, e:
+                messagebox_errors(self, e)
 
             self.state = "EMPTY"
             self.item = None
-            
-            status = load_database(out)
-            
-            if status == False:
-                QMessageBox.critical(None,
-                    self.trUtf8("Mnemosyne"),
-                    self.trUtf8("File doesn't appear to be in "+\
-                                "the correct format."),
-                    self.trUtf8("&OK"), QString(), QString(), 0, -1)
-                self.updateDialog()
+
+            try:
+                load_database(out)
+            except MnemosyneError, e:
+                messagebox_errors(self, e)
                 unpause_thinking()
                 return
 
@@ -207,9 +234,11 @@ class MainDlg(MainFrm):
         pause_thinking()
 
         path = get_config("path")
-        status = save_database(path)
-        if status == False:
-            messageUnableToSave(path)
+        
+        try:
+            save_database(path)
+        except MnemosyneError, e:
+            messagebox_errors(self, e)
 
         unpause_thinking()
 
@@ -222,22 +251,25 @@ class MainDlg(MainFrm):
     def fileSaveAs(self):
         
         pause_thinking()
-        
-        out = unicode(QFileDialog.getSaveFileName(get_config("path"),\
-                                                  "*.mem")).encode("utf-8")
+
+        oldPath = expand_path(get_config("path"))
+        out = unicode(QFileDialog.getSaveFileName(oldPath,\
+                 self.trUtf8("Mnemosyne databases (*.mem)"), self))
+                         
         if out != "":
             
             if out[-4:] != ".mem":
                 out += ".mem"
 
             if os.path.exists(out) and out != get_config("path"):
-                if not queryOverwriteFile(out):
+                if not queryOverwriteFile(self, out):
                     unpause_thinking()
                     return
                 
-            status = save_database(out)
-            if status == False:
-                messageUnableToSave(out)
+            try:
+                save_database(out)
+            except MnemosyneError, e:
+                messagebox_errors(self, e)
                 unpause_thinking()
                 return            
 
@@ -256,9 +288,10 @@ class MainDlg(MainFrm):
         
         from xml.sax import saxutils, make_parser
         from xml.sax.handler import feature_namespaces
-        
-        dlg = ImportDlg(self,"Import",0)
+
+        dlg = ImportDlg(self)
         dlg.exec_loop()
+         
         if self.item == None:
             self.newQuestion()
 
@@ -272,9 +305,12 @@ class MainDlg(MainFrm):
     ##########################################################################
 
     def export(self):
+        
         pause_thinking()
-        dlg = ExportDlg(self,"Export",0)
+
+        dlg = ExportDlg(self)
         dlg.exec_loop()
+                
         unpause_thinking()
         
     ##########################################################################
@@ -297,7 +333,7 @@ class MainDlg(MainFrm):
         
         pause_thinking()
         
-        dlg = AddItemsDlg(self,"Add items",0)
+        dlg = AddItemsDlg(self)
         dlg.exec_loop()
         
         if self.item == None:
@@ -316,7 +352,7 @@ class MainDlg(MainFrm):
         
         pause_thinking()
         
-        dlg = EditItemsDlg(self,"Edit items",0)
+        dlg = EditItemsDlg(self)
         dlg.exec_loop()
         rebuild_revision_queue()
         
@@ -338,7 +374,7 @@ class MainDlg(MainFrm):
         
         pause_thinking()
         
-        self.statusBar().message("Please wait...")
+        self.statusBar().message(self.trUtf8("Please wait..."))
         clean_duplicates(self)
         rebuild_revision_queue()
         
@@ -347,6 +383,19 @@ class MainDlg(MainFrm):
             
         self.updateDialog()
         unpause_thinking()
+
+    ##########################################################################
+    #
+    # showStatistics
+    #
+    ##########################################################################
+    
+    def showStatistics(self):
+        
+        pause_thinking()
+        dlg = StatisticsDlg(self)
+        dlg.exec_loop()
+        unpause_thinking()
         
     ##########################################################################
     #
@@ -357,7 +406,7 @@ class MainDlg(MainFrm):
     def editCurrentItem(self):
         
         pause_thinking()
-        dlg = EditItemDlg(self.item,self,"Edit current item",0)
+        dlg = EditItemDlg(self.item, self)
         dlg.exec_loop()
         self.updateDialog()
         unpause_thinking()
@@ -374,7 +423,7 @@ class MainDlg(MainFrm):
         
         status = QMessageBox.warning(None,
                     self.trUtf8("Mnemosyne"),
-                    self.trUtf8("Delete current item?"),
+                    self.trUtf8("Delete current card?"),
                     self.trUtf8("&Yes"), self.trUtf8("&No"),
                     QString(), 1, -1)
         
@@ -395,7 +444,7 @@ class MainDlg(MainFrm):
         
         pause_thinking()
         
-        dlg = ActivateCategoriesDlg(self,"Activate categories",0)
+        dlg = ActivateCategoriesDlg(self)
         dlg.exec_loop()
         
         rebuild_revision_queue()
@@ -432,8 +481,15 @@ class MainDlg(MainFrm):
     def configuration(self):
         
         pause_thinking()
-        dlg = ConfigurationDlg(self,"Configure Mnemosyne",0)
+        dlg = ConfigurationDlg(self)
         dlg.exec_loop()
+
+        rebuild_revision_queue()
+        if not in_revision_queue(self.item):
+            self.newQuestion()
+        else:
+            remove_from_revision_queue(self.item) # It's already being asked.
+            
         self.updateDialog()
         unpause_thinking()
             
@@ -445,11 +501,15 @@ class MainDlg(MainFrm):
 
     def closeEvent(self, event):
         
-        save_config()
-        status = unload_database()
-        if status == False:
-            messageUnableToSave(get_config("path"))
-        event.accept()
+        try:
+            save_config()
+            backup_database()
+            unload_database()
+        except MnemosyneError, e:
+            messagebox_errors(self, e)
+            event.ignore()
+        else:
+            event.accept()
 
     ##########################################################################
     #
@@ -460,7 +520,20 @@ class MainDlg(MainFrm):
     def productTour(self):
         
         pause_thinking()
-        dlg = ProductTourDlg(self,"Mnemosyne product tour",0)
+        dlg = ProductTourDlg(self)
+        dlg.exec_loop()
+        unpause_thinking()
+        
+    ##########################################################################
+    #
+    # Tip
+    #
+    ##########################################################################
+    
+    def Tip(self):
+        
+        pause_thinking()
+        dlg = TipDlg(self)
         dlg.exec_loop()
         unpause_thinking()
         
@@ -472,13 +545,9 @@ class MainDlg(MainFrm):
     
     def helpAbout(self):
         
-        import mnemosyne.version
         pause_thinking()
-        QMessageBox.about(None,
-            self.trUtf8("Mnemosyne"),
-            self.trUtf8("Mnemosyne " + mnemosyne.version.version + "\n\n"+
-                        "Author: Peter Bienstman\n\n" +
-                        "More info: http://mnemosyne-proj.sourceforge.net\n"))
+        dlg = AboutDlg(self)
+        dlg.exec_loop()
         unpause_thinking()
 
     ##########################################################################
@@ -496,8 +565,10 @@ class MainDlg(MainFrm):
             self.item = get_new_question(learn_ahead)
             if self.item != None:
                 self.state = "SELECT SHOW"
+                self.style = card_props(preprocess(self.item.q))
             else:
                 self.state = "SELECT AHEAD"
+                self.style = card_props("")
 
         self.q_sound_played = False
         self.a_sound_played = False
@@ -526,11 +597,32 @@ class MainDlg(MainFrm):
     ##########################################################################
 
     def gradeAnswer(self, grade):
-
-        process_answer(self.item, grade)
+        
+        interval = process_answer(self.item, grade)
         self.newQuestion()
         self.updateDialog()
 
+        if get_config("show_intervals") == "statusbar":
+            self.statusBar().message(self.trUtf8("Returns in ").append(\
+                str(interval)).append(self.trUtf8(" day(s).")))
+            
+    ##########################################################################
+    #
+    # next_rep_string
+    #
+    ##########################################################################
+
+    def next_rep_string(self, days):
+        
+        if days == 0:
+            return QString('\n') + self.trUtf8("Next repetition: today.")
+        elif days == 1:
+            return QString('\n') + self.trUtf8("Next repetition: tomorrow.")
+        else: 
+            return QString('\n') + self.trUtf8("Next repetition in ").\
+                   append(QString(str(days))).\
+                   append(self.trUtf8(" days."))
+        
     ##########################################################################
     #
     # updateDialog
@@ -542,19 +634,26 @@ class MainDlg(MainFrm):
         # Update caption.
         
         database_name = os.path.basename(get_config("path"))[:-4]
-        caption_text = unicode("Mnemosyne - " + database_name, "utf-8")
+        caption_text = u"Mnemosyne - " + database_name
         self.setCaption(caption_text)
 
         # Update menu bar.
-        
-        if self.item != None and self.state == "SELECT GRADE":
-            self.editCurrentItemAction.setEnabled(True)
+
+        if get_config("only_editable_when_answer_shown") == True:
+            if self.item != None and self.state == "SELECT GRADE":
+                self.editCurrentItemAction.setEnabled(True)
+            else:
+                self.editCurrentItemAction.setEnabled(False)
         else:
-            self.editCurrentItemAction.setEnabled(False)
+            if self.item != None:
+                self.editCurrentItemAction.setEnabled(True)
+            else:
+                self.editCurrentItemAction.setEnabled(False)            
+            
         self.deleteCurrentItemAction.setEnabled(self.item != None)
         self.editItemsAction.setEnabled(number_of_items() > 0)
 
-        # Update tool bar.
+        # Update toolbar.
         
         if get_config("hide_toolbar") == True:
             self.toolbar.hide()
@@ -578,25 +677,41 @@ class MainDlg(MainFrm):
 
         increase_non_latin = get_config("non_latin_font_size_increase")
         non_latin_size = font.pointSize() + increase_non_latin
-
+        
         # Update question and answer alignment.
-
-        # Note: for some reason the default alignment is not AlignCenter
-        # (68) but some other value (2116) which seems to make a difference
-        # in how the widgets are laid out and resized.
         
         if get_config("left_align") == True:
-            alignment = Qt.AlignAuto | Qt.AlignVCenter
+            alignment = Qt.AlignAuto    | Qt.AlignVCenter | Qt.WordBreak
         else:
-            alignment = 2116
+            alignment = Qt.AlignHCenter | Qt.AlignVCenter | Qt.WordBreak
 
         self.question.setAlignment(alignment)
         self.answer.setAlignment(alignment)
 
+        # Hide/show the question and answer boxes
+        
+        if self.state == "SELECT SHOW":
+            self.question.show()
+            self.question_label.show()
+            if self.style["answerbox"] == "overlay":
+                self.answer.hide()
+                self.answer_label.hide()
+        elif self.state == "SELECT GRADE":
+            self.answer.show()
+            self.answer_label.show()
+            if self.style["answerbox"] == "overlay":
+                self.question.hide()
+                self.question_label.hide()
+        else:
+            self.question.show()
+            self.question_label.show()
+            self.answer.show()
+            self.answer_label.show()
+        
         # Update question label.
         
         question_label_text = self.trUtf8("Question:")
-        if self.item != None and self.item.cat.name != "<default>":
+        if self.item!=None and self.item.cat.name!=self.trUtf8("<default>"):
             question_label_text += " " + preprocess(self.item.cat.name)
         self.question_label.setText(question_label_text)
 
@@ -631,33 +746,75 @@ class MainDlg(MainFrm):
                 text = set_non_latin_font_size(text, non_latin_size)
             self.answer.setText(text)
 
-        # Update buttons.
+        # Update 'show answer' button.
         
         if self.state == "EMPTY":
-            show_enabled, default, text = 0, 1, "&Show answer"
+            show_enabled, default, text = 0, 1, self.trUtf8("Show &answer")
             grades_enabled = 0
         elif self.state == "SELECT SHOW":
-            show_enabled, default, text = 1, 1, "&Show answer"
+            show_enabled, default, text = 1, 1, self.trUtf8("Show &answer")
             grades_enabled = 0
         elif self.state == "SELECT GRADE":
-            show_enabled, default, text = 0, 1, "&Show answer"
+            show_enabled, default, text = 0, 1, self.trUtf8("Show &answer")
             grades_enabled = 1
         elif self.state == "SELECT AHEAD":
-            show_enabled, default, text = 1, 0, "Learn ahead of schedule"
+            show_enabled, default, text = 1, 0, \
+                                     self.trUtf8("Learn ahead of schedule")
             grades_enabled = 0
 
-        self.show_button.setText(self.trUtf8(text))
+        self.show_button.setText(text)
         self.show_button.setDefault(default)
         self.show_button.setEnabled(show_enabled)
 
+        # Update grade buttons.
+
+        if self.item != None and self.item.grade in [0,1]:
+            i = 0 # Acquisition phase.
+        else:
+            i = 1 # Retention phase.     
+            
         self.grade_4_button.setDefault(grades_enabled)
         self.grades.setEnabled(grades_enabled)
+        
+        QToolTip.setWakeUpDelay(0)
+
+        for grade in range(0,6):
+
+            # Tooltip.
+            
+            QToolTip.remove(self.grade_buttons[grade])
+            
+            if self.state == "SELECT GRADE" and \
+               get_config("show_intervals") == "tooltips":
+                QToolTip.add(self.grade_buttons[grade],
+                      tooltip[i][grade].
+                      append(self.next_rep_string(process_answer(self.item,
+                                                  grade, dry_run=True))))
+            else:
+                QToolTip.add(self.grade_buttons[grade], tooltip[i][grade])
+
+            # Button text.
+                    
+            if self.state == "SELECT GRADE" and \
+               get_config("show_intervals") == "buttons":
+                self.grade_buttons[grade].setText(\
+                        str(process_answer(self.item, grade, dry_run=True)))
+                self.grades.setTitle(\
+                    self.trUtf8("Pick days until next repetition:"))
+            else:
+                self.grade_buttons[grade].setText(str(grade))
+                self.grades.setTitle(self.trUtf8("Grade your answer:"))
+
+            self.grade_buttons[grade].setAccel(QKeySequence(str(grade)))
 
         # Update status bar.
         
-        self.sched .setText("Scheduled: "     + str(scheduled_items()))
-        self.notmem.setText("Not memorised: " + str(non_memorised_items()))
-        self.all   .setText("All: "           + str(active_items()))
+        self.sched .setText(self.trUtf8("Scheduled: ").append(QString(\
+                            str(scheduled_items()))))
+        self.notmem.setText(self.trUtf8("Not memorised: ").append(QString(\
+                            str(non_memorised_items()))))
+        self.all   .setText(self.trUtf8("All: ").append(QString(\
+                            str(active_items()))))
 
         if self.shrink == True:
             self.adjustSize()
